@@ -2,11 +2,15 @@
 import csv
 import json
 import logging
+import os
 import pickle
 import sys
 
+import botocore.config  # type: ignore
 import smart_open  # type: ignore
 
+import datawelder.readwrite
+
 from typing import (
     Any,
     Callable,
@@ -48,14 +52,12 @@ class AbstractReader:
         field_names: Optional[List[str]] = None,
         fmtparams: Optional[Dict[str, str]] = None,
         types: Optional[List[DataType]] = None,
-        sotparams: Optional[Dict[str, Any]] = None,
     ) -> None:
         self.path = path
         self._key = key
         self.field_names = field_names
         self.fmtparams = fmtparams
         self.types = types
-        self._sotparams = sotparams
 
         if field_names and types and len(field_names) != len(types):
             raise ValueError('field_names and types must be of same length if specified')
@@ -68,7 +70,7 @@ class AbstractReader:
 
     def __enter__(self):
         if self.path is None:
-            self._fin = smart_open.open(self.path, 'r', transport_params=self._sotparams)
+            self._fin = datawelder.readwrite.open(self.path, 'r')
         else:
             self._fin = sys.stdin
         return self
@@ -93,7 +95,7 @@ class CsvReader(AbstractReader):
         if self.path is None:
             self._fin = sys.stdin
         else:
-            self._fin = smart_open.open(self.path, 'r', transport_params=self._sotparams)
+            self._fin = datawelder.readwrite.open(self.path, 'r')
         self._reader = csv.reader(self._fin, **fmtparams)
         self._linenum = 0
 
@@ -158,7 +160,7 @@ class JsonReader(AbstractReader):
         if self.path is None:
             self._fin = sys.stdin.buffer
         else:
-            self._fin = smart_open.open(self.path, 'rb', transport_params=self._sotparams)
+            self._fin = datawelder.readwrite.open(self.path, 'rb')
         return self
 
     def __next__(self):
@@ -279,14 +281,16 @@ def csv_fmtparams(fmtparams: Dict[str, str]) -> Dict[str, Any]:
         'skipinitialspace': bool,
         'strict': bool,
     }
-    scrubbed = {}
+    scrubbed: Dict[str, Any] = {}
     for key, value in fmtparams.items():
         try:
             t = types[key]
         except KeyError:
             _LOGGER.error('ignoring unknown fmtparams key: %r', key)
         else:
-            if t == bool:
+            if t == bool and value in (True, False):
+                scrubbed[key] = value
+            elif t == bool:
                 scrubbed[key] = value.lower() == 'true'
             else:
                 scrubbed[key] = t(value)
@@ -301,7 +305,6 @@ class AbstractWriter:
         field_indices: List[int],
         field_names: List[str],
         fmtparams: Optional[Dict[str, str]] = None,
-        sotparams: Optional[Dict[str, Any]] = None,
     ) -> None:
         """
         :param path: Where to write to.
@@ -316,7 +319,6 @@ class AbstractWriter:
         self._partition_num = partition_num
         self._field_indices = field_indices
         self._field_names = field_names
-        self._sotparams = None
 
         if fmtparams:
             self._fmtparams = fmtparams
@@ -324,7 +326,7 @@ class AbstractWriter:
             self._fmtparams = {}
 
     def __enter__(self):
-        self._fout = smart_open.open(self._path, 'wb', transport_params=self._sotparams)
+        self._fout = datawelder.readwrite.open(self._path, 'wb')
         return self
 
     def __exit__(self, *exc):
@@ -381,7 +383,7 @@ class CsvWriter(AbstractWriter):
 
     def __enter__(self):
         fmtparams = csv_fmtparams(self._fmtparams)
-        self._fout = smart_open.open(self._path, 'w', transport_params=self._sotparams)
+        self._fout = datawelder.readwrite.open(self._path, 'w')
         self._writer = csv.writer(self._fout, **fmtparams)
 
         if self._write_header and self._partition_num == 0:
@@ -437,7 +439,13 @@ def open_reader(
     if fmt != CSV and types:
         raise ValueError('the types parameter is only supported when reading CSV')
 
-    return cls(path, key, field_names, fmtparams, types)
+    return cls(
+        path,
+        key,
+        field_names=field_names,
+        types=types,
+        fmtparams=fmtparams,
+    )
 
 
 def open_writer(
@@ -457,7 +465,7 @@ def open_writer(
     elif fmt == CSV:
         cls = CsvWriter
     else:
-        assert False
+        assert False, 'unknown format: %r' % fmt
 
     return cls(
         path,
@@ -466,3 +474,45 @@ def open_writer(
         field_names=field_names,
         fmtparams=fmtparams,
     )
+
+
+def _inject_parameters(endpoint_url, kwargs):
+    #
+    # transport_params may be set to None or absent altogether
+    #
+    try:
+        transport_params = kwargs['transport_params']
+        transport_params.keys()
+    except (AttributeError, KeyError, TypeError):
+        transport_params = kwargs['transport_params'] = {}
+
+    if transport_params.get('resource'):
+        #
+        # Don't bother injecting the endpoint_url.  We already have a resource
+        # object, and smart_open will ignore endpoint_url in favor of the
+        # resource.
+        #
+        return
+
+    try:
+        resource_kwargs = transport_params['resource_kwargs']
+        resource_kwargs.keys()
+    except (AttributeError, KeyError, TypeError):
+        resource_kwargs = transport_params['resource_kwargs'] = {}
+
+    resource_kwargs['endpoint_url'] = endpoint_url
+    resource_kwargs['config'] = botocore.config.Config(
+        retries={'mode': 'standard', 'max_attempts': 10},
+    )
+
+
+def open(*args, **kwargs):
+    """Wraps smart open and injects the endpoint_url for work under localstack."""
+    try:
+        endpoint_url = os.environ['AWS_ENDPOINT_URL']
+    except KeyError:
+        pass
+    else:
+        _inject_parameters(endpoint_url, kwargs)
+
+    return smart_open.open(*args, **kwargs)
